--------------------------------------------------
-- MuxFir.vhd
-- by Toshio Iwata at digitalfilter.com 2004/09/09

library IEEE;
  use IEEE.std_logic_1164.all;
  use IEEE.std_logic_arith.all;

entity MuxFir is
   Port (
          RST_N : In std_logic;
          MCLK : In std_logic;
          FSCLK : In std_logic;
          XIN : In std_logic_vector(15 downto 0);
          YOUT : Out std_logic_vector(15 downto 0) );
end MuxFir;

architecture RTL of MuxFir is
  type DATAARRAY16 is array (0 to 54) of signed(15 downto 0);
  signal Xn : DATAARRAY16;
  signal Hn : DATAARRAY16;
  signal HnXn : signed(31 downto 0);
  signal HnXn_20b : signed(19 downto 0);
  signal XIN_sig : signed(15 downto 0);
  signal YOUT_sig : signed(21 downto 0);
  signal ADDOUT : signed(21 downto 0);
  signal counter64 : unsigned(5 downto 0);
  signal MuxXn : signed(15 downto 0);
  signal MuxHn : signed(15 downto 0);
  signal fsclk_dly1, fsclk_dly2 : std_logic;
  signal counter64_clr : std_logic;

begin -- MuxFir
-------------------------------------------------------------------
gen_fsclk_dly : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then
    fsclk_dly2 <= '0';
    fsclk_dly1 <= '0';
  elsif( MCLK'event and MCLK = '1' ) then
    fsclk_dly2 <= fsclk_dly1;
    fsclk_dly1 <= FSCLK;
  end if;
end process gen_fsclk_dly;

-------------------------------------------------------------------
  counter64_clr <= '1' when fsclk_dly2 = '0' and fsclk_dly1 = '1' 
              else '0';
-------------------------------------------------------------------
gen_counter64 : process( RST_N, MCLK )
begin
  if( RST_N = '0' ) then
    counter64 <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( counter64_clr = '1' ) then 
      counter64 <= (others => '0');
    else 
      counter64 <= counter64 + 1;
    end if;
  end if;
end process gen_counter64;

-------------------------------------------------------------------
gen_MuxXn : process( Xn, counter64 )
variable tmpint : integer;
begin
  tmpint := conv_integer(counter64);
  if( tmpint < 55 ) then 
    MuxXn <= Xn(tmpint);
  else 
    MuxXn <= (others => '0');
  end if;
end process gen_MuxXn;

-------------------------------------------------------------------
gen_MuxHn : process( Hn, counter64 )
variable tmpint : integer;
begin
  tmpint := conv_integer(counter64);
  if( tmpint < 55 ) then 
    MuxHn <= Hn(tmpint);
  else 
    MuxHn <= (others => '0');
  end if;
end process gen_MuxHn;

-------------------------------------------------------------------
  HnXn <= MuxXn * MuxHn;

-------------------------------------------------------------------
gen_XIN_sig : process( XIN )
begin
  for i in 15 downto 0 loop
    XIN_sig(i) <= XIN(i);
  end loop;
end process gen_XIN_sig;

----------------------------------------------------
gen_Xn : process( RST_N, FSCLK )
begin 
  if( RST_N = '0' ) then
    for i in 54 downto 0 loop
      Xn(i) <= (others => '0');
    end loop;
  elsif( FSCLK'event and FSCLK = '1' ) then
    for i in 54 downto 1 loop
      Xn(i) <= Xn(i-1);
    end loop;
    Xn(0) <= XIN_sig;
  end if;
end process gen_Xn;

----------------------------------------------------
gen_HnXn_20b: process( HnXn )
begin
  for k in 19 downto 0 loop
    HnXn_20b(k) <= HnXn(k+11);
  end loop;
end process gen_HnXn_20b;

----------------------------------------------------
  ADDOUT <= HnXn_20b + YOUT_sig;

----------------------------------------------------
gen_YOUT_sig : process( RST_N, MCLK )
variable tmpint : signed(21 downto 0);
begin
  if( RST_N = '0' ) then
    YOUT_sig <= (others => '0');
  elsif( MCLK'event and MCLK = '1' ) then
    if( counter64_clr = '1' ) then 
      YOUT_sig <= (others => '0');
    else
      YOUT_sig <= ADDOUT;
    end if;
  end if;
end process gen_YOUT_sig;

-------------------------------------------------------------------
gen_YOUT : process( RST_N, FSCLK )
begin
  if( RST_N = '0' ) then
    YOUT <= (others => '0');
  elsif( FSCLK'event and FSCLK = '1' ) then
    if( YOUT_sig(21) = '0' and -- positive value
       ( YOUT_sig(20) = '1' 
      or YOUT_sig(19) = '1' ) ) then -- overflow
      YOUT <= "0111111111111111";
    elsif( YOUT_sig(21) = '1' and -- negative value
       ( YOUT_sig(20) = '0' 
      or YOUT_sig(19) = '0' ) ) then -- underflow
      YOUT <= "1000000000000000";
    else 
      for i in 15 downto 0 loop
        YOUT(i) <= YOUT_sig(i+4);
      end loop;
    end if;
  end if;
end process gen_YOUT;

----------------------------------------------------
  Hn(0) <= "1111111111111111";
  Hn(1) <= "0000000000000000";
  Hn(2) <= "0000000000000110";
  Hn(3) <= "0000000000001110";
  Hn(4) <= "0000000000001101";
  Hn(5) <= "1111111111111010";
  Hn(6) <= "1111111111011000";
  Hn(7) <= "1111111111000100";
  Hn(8) <= "1111111111011110";
  Hn(9) <= "0000000000110000";
  Hn(10) <= "0000000010001100";
  Hn(11) <= "0000000010011101";
  Hn(12) <= "0000000000100100";
  Hn(13) <= "1111111101000110";
  Hn(14) <= "1111111010011000";
  Hn(15) <= "1111111011010001";
  Hn(16) <= "0000000000110010";
  Hn(17) <= "0000001000010011";
  Hn(18) <= "0000001100001100";
  Hn(19) <= "0000000111010101";
  Hn(20) <= "1111111001101100";
  Hn(21) <= "1111101010100011";
  Hn(22) <= "1111100110000100";
  Hn(23) <= "1111110110100101";
  Hn(24) <= "0000011101010101";
  Hn(25) <= "0001001111011010";
  Hn(26) <= "0001111001101101";
  Hn(27) <= "0010001010001111";
  Hn(28) <= "0001111001101101";
  Hn(29) <= "0001001111011010";
  Hn(30) <= "0000011101010101";
  Hn(31) <= "1111110110100101";
  Hn(32) <= "1111100110000100";
  Hn(33) <= "1111101010100011";
  Hn(34) <= "1111111001101100";
  Hn(35) <= "0000000111010101";
  Hn(36) <= "0000001100001100";
  Hn(37) <= "0000001000010011";
  Hn(38) <= "0000000000110010";
  Hn(39) <= "1111111011010001";
  Hn(40) <= "1111111010011000";
  Hn(41) <= "1111111101000110";
  Hn(42) <= "0000000000100100";
  Hn(43) <= "0000000010011101";
  Hn(44) <= "0000000010001100";
  Hn(45) <= "0000000000110000";
  Hn(46) <= "1111111111011110";
  Hn(47) <= "1111111111000100";
  Hn(48) <= "1111111111011000";
  Hn(49) <= "1111111111111010";
  Hn(50) <= "0000000000001101";
  Hn(51) <= "0000000000001110";
  Hn(52) <= "0000000000000110";
  Hn(53) <= "0000000000000000";
  Hn(54) <= "1111111111111111";

end; -- MuxFir
